@@ -71,7 +71,7 @@ qoldlogspline <- function(p, fit)
 
 roldlogspline <- function(n, fit)
 {
-    if(class(fit)!="oldlogspline")
+    if(!inherits(fit, "oldlogspline"))
        stop("fit is not an oldlogspline object")
     pp <- runif(n)
     qoldlogspline(pp, fit)
@@ -80,7 +80,7 @@ roldlogspline <- function(n, fit)
 doldlogspline <- function(q, fit)
 {
     x <- q
-    if(class(fit)!="oldlogspline")
+    if(!inherits(fit, "oldlogspline"))
        stop("fit is not an oldlogspline object")
         q <- unstrip(q)
     y <- fit$coef[1] + x * fit$coef[2]
@@ -100,7 +100,7 @@ doldlogspline <- function(q, fit)
 plot.oldlogspline <- function(x, n = 100, what = "d", xlim, xlab = "", ylab = "", type = "l", add = FALSE, ...)
 {
     fit <- x
-    if(class(fit)!="oldlogspline")
+    if(!inherits(fit, "oldlogspline"))
        stop("fit is not an oldlogspline object")
         if(missing(xlim)) {
                 u1 <- qoldlogspline(0.01, fit)
@@ -136,7 +136,7 @@ print.oldlogspline <- function(x,...)
 }
 summary.oldlogspline <- function(object,...)
 {
-    if(class(object)!="oldlogspline")
+    if(!inherits(object, "oldlogspline"))
        stop("fit is not an oldlogspline object")
     fit <- object
     if(fit$delete==FALSE)stop(paste("summary.oldlogspline can only provide",
@@ -316,6 +316,7 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
 # SorC will carry the error messages - in code form
         SorC <- vector(mode = "integer", length = 35)
         SorC[1] <- 1    # the actual function call
+        SorC[17] <- 0
         nsample[6] <- nsample[6]-1
         if(length(table(sample))<3)stop("Not enough unique values")
         z <- .C("logcensor",
@@ -333,6 +334,27 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
                 as.double(sample),
                 logl = as.double(rep(0, n1 + 1)),
         PACKAGE = "logspline")
+        SorC <- z$SorC  # error messages
+        if(SorC[1] == -1 && SorC[28] == 0 && nsample[1]!=nsample[2] && nsample[2]>15){
+           SorC <- vector(mode = "integer", length = 35)
+           SorC[1] <- 1    # the actual function call
+           SorC[17] <- 1
+           z <- .C("logcensor",
+                   as.integer(delete),
+                   as.integer(iautoknot),
+                   as.double(sample),
+                   as.integer(nsample),
+                   bd = as.double(xbound),
+                   SorC = as.integer(SorC),
+                   nk = as.integer(nknots),
+                   kt = as.double(knots),
+                   cf = as.double(c(knots, 0, 0)),
+                   as.double(penalty),
+                   as.double(sample),
+                   as.double(sample),
+                   logl = as.double(rep(0, n1 + 1)),
+           PACKAGE = "logspline")
+        }
         bound <- c(z$bd[2], z$bd[3], z$bd[4], z$bd[5])
         SorC <- z$SorC  # error messages
         if(abs(SorC[1]) > 2) {
@@ -532,7 +554,7 @@ logspline <- function(x, lbound, ubound, maxknots=0, knots, nknots=0,
 }
 plogspline <- function(q, fit)
 {
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
    if(!missing(q))q <- unstrip(q)
     sq <- rank(q)
@@ -553,7 +575,7 @@ plogspline <- function(q, fit)
 }
 qlogspline <- function(p, fit)
 {
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
    if(!missing(p))p <- unstrip(p)
     sp <- rank(p)
@@ -574,29 +596,30 @@ qlogspline <- function(p, fit)
 }
 rlogspline <- function(n, fit)
 {
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
     pp <- runif(n)
     qlogspline(pp, fit)
 }
-dlogspline <- function(q, fit)
+dlogspline <- function(q, fit, log = FALSE)
 {
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
    if(!missing(q))q <- unstrip(q)
     x <- q
     y <- fit$coef.pol[1] + x * fit$coef.pol[2]
     for(i in 1:length(fit$knots)) 
        y <- y + fit$coef.kts[i] * ((abs(x - fit$knots[i]) +x- fit$knots[i])/2)^3
-    y <- exp(y)
-    if(fit$bound[1] > 0) y[x < fit$bound[2]] <- 0
-    if(fit$bound[3] > 0) y[x > fit$bound[4]] <- 0
+    if(fit$bound[1] > 0) y[x < fit$bound[2]] <- -Inf
+    if(fit$bound[3] > 0) y[x > fit$bound[4]] <- -Inf
+    if (!log)
+       y <- exp(y)
     y
 }
 plot.logspline <-function(x, n = 100, what = "d", add = FALSE, xlim, xlab = "", ylab = "", type = "l", ...)
 {
         fit <- x
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
         if(add){
                 plim <- (par()$usr)[1:2]
@@ -638,7 +661,7 @@ print.logspline <- function(x,...)
 summary.logspline <- function(object,...)
 {
         fit <- object
-    if(class(fit)!="logspline")
+   if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
    ul <- fit$penalty
    um <- fit$samples[1]
